#pragma once

#include "Betriebssystem.h"
#include "ReferenceCounter.h"

#ifdef WIN32
struct ID3D11Buffer;
struct D3D11_BUFFER_DESC;
struct ID3D11ShaderResourceView;
struct ID3D11Device;
struct ID3D11DeviceContext;
#endif

namespace Framework
{
    class DX12CopyCommandQueue;
    class DX12DirectCommandQueue;

    //! Eine Schnittstelle zwischen dem Arbeitsspeicher und dem Grafikspeicher
    class DXBuffer : public virtual ReferenceCounter
    {
    protected:
        void* data;
        bool changed;
        int len;
        int elLen;

    public:
        //! Konstruktor
        //! \param bind Der verwendungszweck des Buffers. Beispiel: D3D11_BIND_INDEX_BUFFER, D3D11_BIND_VERTEX_BUFFER.
        //! \param eLen Lnge eines einzelnen Elements in Bytes
        DLLEXPORT DXBuffer( int eLen );
        //! Destruktor
        DLLEXPORT virtual ~DXBuffer();
        //! Setzt den gendert flg, so das beim nchsten auruf von 'kopieren' die daten neu kopiert werden
        DLLEXPORT void setChanged();
        //! ndert die lnge des Buffers beim nchsten aufruf von 'kopieren'
        //! \param len Die Lnge in Bytes
        DLLEXPORT void setLength( int len );
        //! Legt fest, was beim nchsten aufruf von 'kopieren' kopiert wird
        //! \param data Ein zeiger auf die Daten
        DLLEXPORT void setData( void* data );
        //! Kopiert die Daten in den Buffer, fals sie sich verndert haben
        DLLEXPORT virtual void copieren( int byteCount = -1 ) = 0;
        //! Gibt die Lnge eines Elementes in bytes zurck
        DLLEXPORT int getElementLength() const;
        //! Gibt die Anzahl der Elemente im Buffer zurck
        DLLEXPORT int getElementAnzahl() const;
    };

#ifdef WIN32
    //! Ein Buffer mit Daten im Grafikspeicher
    class DX11Buffer : public DXBuffer
    {
    protected:
        D3D11_BUFFER_DESC* description;
        ID3D11Buffer* buffer;
        ID3D11Device* device;
        ID3D11DeviceContext* context;
    public:
        //! Konstruktor
        //! eSize: Die Lnge eines Elementes in Bytes
        DLLEXPORT DX11Buffer( int eSize, ID3D11Device* device, ID3D11DeviceContext* context, int bindFlags );
        //! Destruktor
        DLLEXPORT virtual ~DX11Buffer();
        //! Kopiert die Daten in den Buffer, fals sie sich verndert haben
        DLLEXPORT void copieren( int byteCount = -1 ) override;
        //! Gibt den Buffer zurck
        DLLEXPORT ID3D11Buffer* zBuffer() const;
    };

    //! Ein Buffer von Indizes aus dem Buffer mit Eckpunkten, wovon immer drei ein Dreieck ergeben, das gezeichnet wird
    class DX11StructuredBuffer : public DX11Buffer
    {
    private:
        ID3D11ShaderResourceView* view;
    public:
        //! Konstruktor
        //! eSize: Die Lnge eines Elementes in Bytes
        DLLEXPORT DX11StructuredBuffer( int eSize, ID3D11Device* device, ID3D11DeviceContext* context );
        //! Destruktor
        DLLEXPORT virtual ~DX11StructuredBuffer();
        //! Kopiert die Daten in den Buffer, fals sie sich verndert haben
        DLLEXPORT void copieren( int byteCount = -1 ) override;
        //! Gibt die verwendtete Shader Resource View zurck
        DLLEXPORT operator ID3D11ShaderResourceView* () const;
    };
#endif
}