#ifndef DateiSystem_H
#define DateiSystem_H

#include <fstream>
#include "Array.h"
#include "Punkt.h"

namespace Framework
{
    class Bild; //! Bild.h
    class Text; //! Text.h
    class FBalken; //! Fortschritt.h
#ifdef WIN32
    class Buchstabe; //! Schrift.h
    class Alphabet; //! Schrift.h
    class Schrift; //! Schrift.h
#endif
    class LTDBPixel; //! aus dieser Datei
    class LTDBKopf; //! aus dieser Datei
    class LTDBBody; //! aus dieser Datei
    class LTDBDatei; //! aus dieser Datei
    class LTDSPixel; //! aus dieser Datei
#ifdef WIN32
    class LTDSDateiKopf; //! aus dieser Datei
    class LTDSSchriftKopf; //! aus dieser Datei
    class LTDSBuchstabenKopf; //! aus dieser Datei
    class LTDSBuchstabenKrper; //! aus dieser Datei
    class LTDSDatei; //! aus dieser Datei
#endif

    //! LTDB Dateivormat --- Dient zum speichern von mehreren Bildern in einer Datei.

    //! Dient zum Speichern und Laden eines einzelnen Pixels aus einem Bild im LTDB Dateiformat
    class LTDBPixel : public virtual ReferenceCounter //! Pixel einer LTDB Datei
    {
    private:
        LTDBPixel *davor; //! Letzter Pixel
        char index; //! Bitlnge des Pixels
        char iR, iG, iB, iA;
        char miR, miG, miB, miA;
        char maxIndex; //! Lnge des Pixels
        bool change : 1; //! Verndert sich etwas an den volgenden 5 Variablen
        bool changeR : 1; //! ndert sich Rot
        bool changeG : 1; //! ndert sich Grn
        bool changeB : 1; //! ndert sich Blau
        bool changeA : 1; //! ndert sich Alpha
        unsigned char komp : 3; //! Komprimierung der Farbwerte
        unsigned char R; //! Rot
        unsigned char G; //! Grn
        unsigned char B; //! Blau
        unsigned char A; //! Alpha
        bool addBitZuFarbe( unsigned char bit ); //! Fgt den Farbwerten ein Bit hinzu
        bool getNextFarbeBit( char &byte, int i ); //! Speichert das nchste Farbbit in byte

    public:
        //! Konstruktor
        //! \param davor Der Pixel, der Vorher geladen wurde. 0, falls die der Erste Pixel ist
        DLLEXPORT LTDBPixel( LTDBPixel *davor );
        //! Destruktor 
        DLLEXPORT ~LTDBPixel();
        //! zum Laden gedacht. Fgt dem Pixel einiege bits hinzu
        //! \param byte Das zuletzt aus der Datei gelesene Byte
        //! \param begin Der Index des ersten Bits im byte, wo der Pixel beginnt
        //! \return Der Index des Bits im Byte, wo der Pixel aufgehhrt hat. -1, falls der Pixel am ende Des Bytes noch nicht zuende ist
        DLLEXPORT char addByte( char byte, char begin );
        //! zum speichern gedacht. Setzt die Farbe, die im Pixel gespeichert werden soll
        //! \param f Die zu speichernde Farbe
        DLLEXPORT void setFarbe( int f );
        //! Komprimiert den Pixel. Muss vor dem Speichern aufgerufen werden.
        DLLEXPORT void komprimieren();
        //! Gibt ein Teil der Bits zurck, die den Pixel representieren
        //! \param byte Eine Referens auf das Byte, dass als nchstes gespeichert werden soll
        //! \param begin Der Index des ersten Bits im zu speichernden Byte, wo der Pixel gespeichert werden soll
        //! \return Der Index des Bits im zu speichernden Byte, wo der Pixel aufhhrt. -1, falls der Pixel im nchsten Byte fortgesetzt werden muss
        DLLEXPORT char getNextByte( char &byte, int begin );
        //! Gibt den Farbwert des Pixels zurck
        DLLEXPORT int zuFarbe() const;
        //! Gibt zurck, ob sich der Anteil an Rot in der Farbe im Vergleich zum Pixel davor gendert hat
        DLLEXPORT bool getChangeR() const;
        //! Gibt zurck, ob sich der Anteil an Grn in der Farbe im Vergleich zum Pixel davor gendert hat
        DLLEXPORT bool getChangeG() const;
        //! Gibt zurck, ob sich der Anteil an Blau in der Farbe im Vergleich zum Pixel davor gendert hat
        DLLEXPORT bool getChangeB() const;
        //! Gibt zurck, ob sich der Anteil an Alpha in der Farbe im Vergleich zum Pixel davor gendert hat
        DLLEXPORT bool getChangeA() const;
        //! Gibt die Komprimierung des Pixels zurck
        DLLEXPORT unsigned char getKomp() const;
        //! Gibt den Anteil an Rot in der Farbe des Pixels zurck
        DLLEXPORT unsigned char getR() const;
        //! Gibt den Anteil an Grn in der Farbe des Pixels zurck
        DLLEXPORT unsigned char getG() const;
        //! Gibt den Anteil an Blau in der Farbe des Pixels zurck
        DLLEXPORT unsigned char getB() const;
        //! Gibt den Anteil an Alpha in der Farbe des Pixels zurck
        DLLEXPORT unsigned char getA() const;
    };

    //! Der Kopf des LTDB Dateiformates. Hier werden informationen ber alle Abgespeicherten Bilder verwaltet
    class LTDBDateiKopf : public virtual ReferenceCounter
    {
    private:
        RCArray< Text > *bilder;
        Array< __int64 > *pos;
        int bAnzahl;

    public:
        //! konstructor
        DLLEXPORT LTDBDateiKopf();
        //! destructor 
        DLLEXPORT ~LTDBDateiKopf();
        //! Entfernt ein Bild aus dem Dateikopf
        //! \param i Der Index des Bildes, welches entfernt werden soll
        DLLEXPORT void removeBild( int i );
        //! Entfernt ein Bild aus dem Dateikopf
        //! \param txt Der Name des Bildes, welches entfernt werden soll
        DLLEXPORT void removeBild( Text *txt );
        //! Fgt dem Dateikopf ein Bild hinzu
        //! \param txt Der Name des Bildes.
        DLLEXPORT void addBild( Text *txt );
        //! Legt den Index des Bytes aus der Datei fest, wo das Bild anfngt
        //! \param i Der index Des Bildes
        //! \param pos Die Position des Bildes in der Datei
        DLLEXPORT void setBildPos( int i, __int64 pos );
        //! Legt den Index des Bytes aus der Datei fest, wo das Bild anfngt
        //! \param txt Der Name Des Bildes
        //! \param pos Die Position des Bildes in der Datei
        DLLEXPORT void setBildPos( Text *txt, __int64 pos );
        //! Ldt den Dateikopf einer LTDB Datei
        //! \param f Ein Zeiger auf einen Fortschrittsbalken, der zum Laden verwendet werden soll. Kann 0 sein.
        //! \param inF Der geffnete ifstream der LTDB Datei, bei dem die Leseposition bereits auf das erste Byte des Dateiopfes zeigt.
        DLLEXPORT void laden( FBalken *f, std::ifstream *inF );
        //! Speichert den Dateikopf in einer LTDB Datei
        //! \param outF Der geffnete ofstream der LTDB Datei, bei dem die Schreibposition bereits auf das erste Byte des Dateikopfes zeigt.
        DLLEXPORT void speichern( std::ofstream *outF ) const;
        //! Gibt den Namen eines bestimmten Bildes zurck
        //! \param i Der Index des Bildes, dessen Namen zurckgegeben werden soll
        //! \return Der Name des bildes
        DLLEXPORT Text *getBild( int i ) const;
        //! Gibt den Namen eines bestimmten Bildes zurck
        //! \param i Der Index des Bildes, dessen Namen zurckgegeben werden soll
        //! \return Der Name des bildes ohne erhhten Reference Counter
        DLLEXPORT Text *zBild( int i ) const;
        //! Gibt den Index des Ersten Bytes eines Bildes in der Datei zurck
        //! \param txt Der Name des Bildes, dessen Beginn gefunden werden soll
        //! \return -1, falls das Bild nicht gefunden wurde.
        DLLEXPORT __int64 getBildPosition( Text *txt ) const;
        //! Gibt den Index des Ersten Bytes eines Bildes in der Datei zurck
        //! Fals der Index nicht existiert wird die Exception std::out_of_range geworfen.
        //! \param indes Der Indes des Bildes, dessen Beginn gefunden werden soll
        DLLEXPORT __int64 getBildPosition( int index ) const;
        //! Gibt den Index eines Bestimmten Bildes zurck
        //! \param txt Der Name des Bildes
        //! \return -1, falls das Bild nicht gefunden wurde
        DLLEXPORT int getBildIndex( Text *txt ) const;
        //! Gibt die Anzahl der Bilder in der Datei zurck
        DLLEXPORT int getbAnzahl() const;
        //! Gibt eine Liste mit Bildern in der Datei ohne erhhten Reference Counter zurck.
        //! Die Liste sollte nicht verndert werden
        DLLEXPORT RCArray< Text > *zBildListe() const;
    };

    //! Die im LTDB Dateikopf gespeicherten Informationen fr ein einzelnes Bild
    class LTDBKopf : public virtual ReferenceCounter
    {
    private:
        __int64 a; //! Det LTDB Dateikopf ist maximal 104 Bits lang
        __int32 b; //! - bis zu 75 Bits fr den Titel
        __int8 c;  //! - 12 Bits fr breite

    public:
        //! Konstruktor 
        DLLEXPORT LTDBKopf();
        //! Lhdt die Informationen ber ein Bild
        //! \param f Der geffnette und an die richtiege Stelle zeigende ifstream der LTDB Datei
        DLLEXPORT void laden( std::ifstream *f );
        //! Setzt die Informationen die gespeichert werden sollen
        //! \param titel Der Titel des Bildes
        //! \param size Die Gre des Bildes
        //! \return Die Anzahl der Buchstaben aus dem Titel, die im LTDB Dateiformat nicht gespeichert werden knnen. Erlaubt ist nur a-z und A-Z und     und    und .
        //! Alle grobuchstaben im Titel werden in Kleinbuchstaben umgewandelt
        DLLEXPORT int Init( Text *titel, const Punkt &size );
        //! Lhd informationen aus geladenen Bits. Wird von der laden( std::ifstream ) Funktion verwendet.
        //! \param BeginBit Der Index des ersten Bits, welches ausgewertet werden soll
        //! \param EndBit Der Index des letzten Bits, welches nichtmehr ausgewertet werden soll
        //! \param bits Die Bits, von denen alle von BeginBit bis EndBit ausgewertet werden sollen
        //! Insgesamt mssen 104 Bits gesetzt werden. Hierauf bezihen sich BeginBit und EndBit
        DLLEXPORT void setBits( int BeginBit, int EndBit, __int16 bits );
        //! Speichert die Informationen in eine Datei
        //! \param f Der geffnete und an die richtiege Stelle zeigende ofstream der LTDB Datei
        DLLEXPORT void speichern( std::ofstream *f ) const;
        //! Gibt die Lnge des Titels zurck
        DLLEXPORT int getTitelLength() const;
        //! Gibt den Titel des Bildes zurck
        DLLEXPORT Text *getTitel() const;
        //! Gibt die Gre des Bildes zurck
        DLLEXPORT Punkt getSize() const;
        //! Gibt die nchsten zu speichernden Bits zurck
        //! \param begin Der Index des ersten Bits, in das gespeichert werden soll
        //! \param end Der Index des letzten Bits, in das gespeichert werden soll
        //! \return 16 Bits, in denen die Informationen zwischen begin und end stehen
        //! Insgesamt mssen 104 Bits gelesen werden. Hierauf bezihen sich BeginBit und EndBit
        DLLEXPORT __int16 getBits( int begin, int end )const;
    };

    //! Verwaltet die Pixeldaten eines einzelnen Bildes einer LTDB Datei
    class LTDBBody : public virtual ReferenceCounter
    {
    private:
        Punkt gr;
        Bild *b;
        int dateiSize;

    public:
        //! Konstruktor 
        DLLEXPORT LTDBBody();
        //! Konstruktor
        //! \param k Der LTDB Kopf des Bildes, der Informationen ber die Gre des Bildes enthlt
        DLLEXPORT LTDBBody( LTDBKopf *k );
        //! Destruktor 
        DLLEXPORT ~LTDBBody();
        //! Setzt die Informationen ber die Gre des Bildes. Wird zum Laden bentigt.
        //! \param k Der LTDB Kopf des Bildes
        DLLEXPORT void init( LTDBKopf k );
        //! Setzt die Informationen ber die Gre des Bildes. Wird zum Laden bentigt.
        //! \param k Der LTDB Kopf des Bildes
        DLLEXPORT void init( LTDBKopf *k );
        //! Ldt die Pixeldaten aus der Datei
        //! \param zF Ein Fortschrittsbalken, der 0 sein kann
        //! \param inF Der geffnete und an die richtiege Stelle zeigende ifstream der LTDB Datei
        DLLEXPORT void laden( FBalken *zF, std::ifstream *inF );
        //! Setzt das Bild, welches gespeichert werden soll
        //! \param b Das zu speichernde Bild
        DLLEXPORT void setBild( Bild *b );
        //! Speichert die Pixeldaten des Bildes in einer LTDB Datei
        //! \param zF Ein Fortschrittsbalken, der 0 sein kann
        //! \param outF Der geffnete und an die richtiege Stelle zeigende ofstream der LTDB Datei
        DLLEXPORT void speichern( FBalken *zF, std::ofstream *outF ) const;
        //! Gibt das geladene Bild zurck
        DLLEXPORT Bild *getBild() const;
        //! Gibt die Gre des Bildes zurck
        DLLEXPORT const Punkt &getSize() const;
    };

    //! Verwaltet eine LTDB Datei
    class LTDBDatei : public virtual ReferenceCounter
    {
    private:
        Text *pfad;
        LTDBDateiKopf *datKpf;

    public:
        //! Konstruktor 
        DLLEXPORT LTDBDatei();
        //! Destruktor 
        DLLEXPORT ~LTDBDatei();
        //! Setzt den Pfad zu Datei
        //! \param pfad Der Pfad
        DLLEXPORT void setDatei( Text *pfad );
        //! Erstellt eine neue LTDB Datei
        DLLEXPORT void erstellen();
        //! Liet grundlegende Informationen aus der Datei.
        //! Wird bentigt, bevor mit der Datei gearbeitet wird
        //! \param zF Ein Fortschrittsbalken, der 0 sein kann
        DLLEXPORT void leseDaten( FBalken *zF );
        //! Lscht die LTDB Datei
        DLLEXPORT void remove();
        //! Lscht ein Bild aus der LTDB Datei
        //! \param zF Ein Fortschrittsbalken der 0 sein kann
        //! \param name Der Name des zu lschenden Bildes
        DLLEXPORT void remove( FBalken *zF, Text *name );
        //! Ldt ein Bild aus der LTDB Datei
        //! \param zF Ein Fortschrittsbalken, der 0 sein kann
        //! \param name Der Name des Bildes, welches geladen werden soll
        //! \return Das geladene Bild. 0, falls das Bild nicht gefunden wurde
        DLLEXPORT Bild *laden( FBalken *zF, Text *name );
        //! Speichert ein neues Bild in der LTDB Datei
        //! \param zF Ein Fortscrittsbalken, der 0 sein kann
        //! \param bild Das Bild, welches gelscht werden soll
        //! \param name Der Name, unter dem das Bild gespeichert werden soll
        //! \return Anzahl der Warnungen, die beim Konvertieren des Namens in einen gltigen Namen aufgetreten sind. -1, falls bereis ein Bild mit dem selben Namen existiert
        DLLEXPORT int speichern( FBalken *zF, Bild *bild, Text *name );
        //! Gibt eine Liste mit gespeicherten Bildern zurck
        //! Die Liste sollte nicht verndert werden
        DLLEXPORT RCArray< Text > *zBildListe();
        //! Gibt den Pfad zur LTDB Datei zurck
        DLLEXPORT Text *getPfad() const;
        //! Gibt die Anzahl der Bilder in der LTDB Datei zurck
        DLLEXPORT int getBildAnzahl() const;
        //! Prft, ob die LTDB Datei existiert
        DLLEXPORT bool istOffen() const;
    };

#ifdef WIN32
    //! LTDS Dateivormat --- Dient zum speichern von Schriftarten

    //! Verwaltet einen einzelnen Pixel von einem Buchstaben
    class LTDSPixel : public virtual ReferenceCounter
    {
    private:
        char index;
        char iA;
        char miA;
        char maxIndex; //! Lnge des Pixels
        bool nder;
        bool nderA;
        unsigned char komp : 3; //! Komprimierung der Farbwerte
        unsigned char alpha;
        LTDSPixel *davor;
        bool addBitZuFarbe( unsigned char bit ); //! Fgt den Farbwerten ein Bit hinzu
        bool getNextFarbeBit( char &byte, int i ); //! Speichert das nchste Farbbit in byte

    public:
        //! Konstruktor
        //! \param davor Der Pixel, der vor diesem geladen wurde. 0, fals die der erste Pixel ist
        DLLEXPORT LTDSPixel( LTDSPixel *davor );
        //! Destruktor 
        DLLEXPORT ~LTDSPixel();
        //! Fgt dem Pixel einiege geladene Bits hinzu. Zum Laden gedacht.
        //! \param byte Das letzte aus der Datei geladene Byte.
        //! \param begin Der Index des ersten Bits im Byte, wo der Pixel beginnt
        //! \return Der Index des Letzten Bits in Byte, wo der Pixel aufhhrt. -1, falls der Pixel im nchsten Byte weitergeht
        DLLEXPORT char addByte( char byte, char begin );
        //! Setzt den Alpha Wert des Pixels. Zum speichern gedacht.
        //! \param alpha Der Alpha Wert des Pixels.
        DLLEXPORT void setAlpha( unsigned char alpha );
        //! Komprimiert den Pixel. Muss vor dem Speichern aufgerufen werden.
        DLLEXPORT void Komp();
        //! Gibt ein Teil der Bits zurck, die den Pixel representieren
        //! \param byte Eine Referens auf das Byte, dass als nchstes gespeichert werden soll
        //! \param begin Der Index des ersten Bits im zu speichernden Byte, wo der Pixel gespeichert werden soll
        //! \return Der Index des Bits im zu speichernden Byte, wo der Pixel aufhhrt. -1, falls der Pixel im nchsten Byte fortgesetzt werden muss
        DLLEXPORT char getNextByte( char &byte, int bbegin );
        //! Gibt die Komprimierung des Pixels zurck
        DLLEXPORT unsigned char getKomp() const;
        //! Gibt zurck, ob sich der Alpha Wert im Vergleich zu dem Pixel davor gendert hat
        DLLEXPORT bool getnderA() const;
        //! Gibt den Alpha Wert des Pixels zurck
        DLLEXPORT unsigned char getA() const;
    };

    //! Verwaltet den Kopf einer LTDS Datei. Er enthlt Informationen ber die in der Datei gespeicherten Schriftgren
    class LTDSDateiKopf : public virtual ReferenceCounter
    {
    private:
        unsigned char sganzahl;
        unsigned char *gr;
        int *pos;

    public:
        //! Konstruktor 
        DLLEXPORT LTDSDateiKopf();
        //! Destruktor 
        DLLEXPORT ~LTDSDateiKopf();
        //! Ldt den Kopf aus der LTDS Datei
        //! \param inF Der geffnete und an die richtige Stelle zeigende ifstream der LTDS Datei
        DLLEXPORT void laden( std::ifstream *inF );
        //! Fgt eine Schriftgre hinzu
        //! \param sg Die Schriftgre, die hinzugefgt werden soll
        DLLEXPORT void addSG( char sg );
        //! Lscht eine Schriftgre
        //! \param sg Die Schriftgre, die gelscht werden soll
        DLLEXPORT void removeSG( char sg );
        //! Speichert den LTDS Kopf in der Datei
        //! \param outF Der geffnete und an die richtiege Stelle zeigende ofstream der LTDS Datei
        DLLEXPORT void speichern( std::ofstream *outF ) const;
        //! Gibt einen Array von gespeicherten Schriftgren zurck
        //! Der Array sollte nicht verdert werden
        DLLEXPORT unsigned char *getSchriftGreList() const;
        //! Gibt einen Array mit positionen der ersten Bytes fr jede Schriftgre aus der Datei zurck
        DLLEXPORT int *getPositionList() const;
        //! Gibt die Anzahl der gespeicherten Schriftgren zurck
        DLLEXPORT int getSchriftGreAnzahl() const;
    };

    //! Der Kopf einer Schriftgre. Enthlt Informationen ber die gespeicherten Zeichen
    class LTDSSchriftKopf : public virtual ReferenceCounter
    {
    private:
        unsigned char schriftSize;
        unsigned char *zeichen;
        int *pos;
        unsigned char zeichenAnzahl;

    public:
        //! Konstruktor 
        DLLEXPORT LTDSSchriftKopf();
        //! Destruktor 
        DLLEXPORT ~LTDSSchriftKopf();
        //! Ldt den Kopf einer Schriftgre aus der Datei
        //! \param inF Der geffnete und an de richtiege Stelle zeigende ifstream der LTDS Datei
        DLLEXPORT void laden( std::ifstream *inF );
        //! Setzt die Schriftgre. Zum speichern gedacht
        //! \param gr Die Schriftgre
        DLLEXPORT void setSchriftgre( unsigned char gr );
        //! Setzt das Alphabet, welches in der Schriftgre gespeichert werden soll
        //! \param alphabet Das Alphabet, welches alle zu speichernden Zeichen in der Schriftgre enthlt
        DLLEXPORT void setZeichenAlphabet( Alphabet *alphabet );
        //! Fgt der Schriftgre ein Zeichen hinzu, welches gespeichert werden soll
        //! \param zeichen Der ASCII code des Zeichens, welches hinzugefgt werden soll
        DLLEXPORT void addZeichen( unsigned char zeichen );
        //! Lscht ein Zeichen aus der Schriftgre
        //! \param zeich Der ASCII code des Zeichens, welches gelscht werden soll
        DLLEXPORT void removeZeichen( unsigned char zeich );
        //! Speichert den Kopf der Schriftgre in der LTDS Datei
        //! \param outF Der geffnete und an die richtiege Stelle zeigende ofstream der LTDS Datei
        DLLEXPORT void speichern( std::ofstream *outF ) const;
        //! Gibt die Schriftgre zurck, zu der dieser Kopf gehhrt
        DLLEXPORT unsigned char getSchriftGre() const;
        //! Gibt die Anzahl der in der Schriftgre gespeicherten Zeichen zurck
        DLLEXPORT unsigned char getZeichenAnzahl() const;
        //! Gibt einen Array mit den Positionen der ersten Bytes von den gespeicherten Zeichen aus der LTDS Datei zurck 
        DLLEXPORT int *getPositionen() const;
        //! Gibt einen Array mit den ASCII codes der Gespeicherten Zeichen zurck
        DLLEXPORT unsigned char *getZeichen() const;
    };

    //! Der Kopf eines einzelnen Zeichens aus der LTDS Datei. Enthlt informationen ber die Pixelgre des Zeichens
    class LTDSBuchstabenKopf : public virtual ReferenceCounter
    {
    private:
        unsigned char zeichen;
        Punkt size;

    public:
        //! Konstruktor 
        DLLEXPORT LTDSBuchstabenKopf();
        //! Ldt die Daten aus der LTDS Datei
        //! \param inF Der geffnete und an die richtiege Stelle zeigende ifstream der LTDS Datei
        DLLEXPORT void laden( std::ifstream *inF );
        //! Setzt die Daten, die gespeichert werden sollen.
        //! \param zeichen Der ASCII code des Zeichens
        //! \param gre Die Gre des Zeichens in Pixeln
        DLLEXPORT void init( unsigned char zeichen, const Punkt &gre );
        //! Setzt die Daten, die gespeichert werden sollen.
        //! \param zeichen Der ASCII code des Zeichens
        //! \param br Die Breite des Zeichens in Pixeln
        //! \param h Die Hhe des Zeichens in Pixeln
        DLLEXPORT void init( unsigned char zeichen, int br, int h );
        //! Speichert die Daten in der LTDS Datei
        //! \param outF Der geffnete und auf die richtiege Stelle zeigende ofstream der LTDS Datei
        DLLEXPORT void speichern( std::ofstream *outF ) const;
        //! Gibt den ASCII code des Zeichens zurck
        DLLEXPORT unsigned char getZeichen() const;
        //! Gibt die Breite des Zeichens in Pixeln zurck
        DLLEXPORT int getBreite() const;
        //! Gibt die Hhe des Zeichens in Pixeln zurck
        DLLEXPORT int getHhe() const;
        //! Gib t die Gre des Zeichens in Pixeln zurck
        DLLEXPORT const Punkt &getGre() const;
    };

    //! Verwaltet die Pixeldaten eines Zeichens
    class LTDSBuchstabenKrper : public virtual ReferenceCounter
    {
    private:
        Punkt size;
        unsigned char zeichen;
        Buchstabe *buchstabe;

    public:
        //! Konstruktor
        //! \param kopf Der Kopf des Zeichens
        DLLEXPORT LTDSBuchstabenKrper( LTDSBuchstabenKopf *kopf );
        //! Destruktor 
        DLLEXPORT ~LTDSBuchstabenKrper();
        //! Setzt den Buchstaben, der gespeichert werden soll
        //! \param zeichen Der zu speichernde Buchstabe
        DLLEXPORT void setBuchstabe( Buchstabe *zeichen );
        //! Ldt die Pixel aus der LTDS Datei
        //! \param inF Der geffnete und auf die richtiege Stelle zeigende ifstream der LTDS Datei
        DLLEXPORT void laden( std::ifstream *inF );
        //! Speichert die Pixel in die LTDS Datei
        //! \param outF der geffnete und an die richtiege Stelle zeigende ofstream der LTDS Datei
        DLLEXPORT void speichern( std::ofstream *outF ) const;
        //! Gibt den geladenen Buchstaben zurck
        DLLEXPORT Buchstabe *getBuchstabe() const;
        //! Gibt den ASCII code des Buchstabens zurck
        DLLEXPORT unsigned char getZeichen() const;
    };

    //! Verwaltet eine LTDS Datei
    class LTDSDatei : public virtual ReferenceCounter
    {
    private:
        Text *pfad;
        LTDSDateiKopf *dateiKopf;

    public:
        //! Konstruktor 
        DLLEXPORT LTDSDatei();
        //! Destruktor 
        DLLEXPORT ~LTDSDatei();
        //! Setzt den Pfad zur Datei
        //! \param txt Der Pfad
        DLLEXPORT void setPfad( Text *txt );
        //! Ldt wichtiege Informationen aus der Datei. Muss vor dem Verwenden der Datei aufgerufen werden
        DLLEXPORT void leseDaten();
        //! Fgt der Datei eine Schriftgre hinzu, fals diese noch nicht existiert
        //! \param alphabet Das Alphabet, welches die Zeichen in der gewnschten Schriftgr0e enthlt
        DLLEXPORT void addSchriftgre( Alphabet *alphabet );
        //! Fgt einer Schriftgre einen Buchstaben hinzu
        //! \param gr Die Schriftgre des Buchstabens
        //! \param zeich Der Buchstabe, der gespeichert werden soll
        //! \param zeichen Der ASCII code des Buchstabens
        DLLEXPORT void addBuchstabe( int gr, Buchstabe *zeich, unsigned char zeichen );
        //! Lscht eine bestimmte Schriftgre aus der Datei
        //! \param gr Die Schriftgre, die entfernt werden soll
        DLLEXPORT void lscheSchrifrGre( int gr );
        //! Lscht einen Buchstaben aus einer Schriftgre
        //! \param gr Die Schriftgre, aus der der Buchstabe entfernt werden soll
        //! \param zeichen Der ASCII code des Zeichens, welches gelscht werden soll
        DLLEXPORT void lscheBuchstabe( int gr, unsigned char zeichen );
        //! Lscht die LTDS Datei
        DLLEXPORT void lscheDatei();
        //! Erstellt die LTDS Datei
        DLLEXPORT void erstelleDatei();
        //! Speichert eine gesammte Schrift in der Datei
        //! \param schrift Die schrift, die gespeichert werden soll
        DLLEXPORT void speicherSchrift( Schrift *schrift );
        //! Ldt die gesammte Schrift aus der Datei
        //! \return Die geladene Schrift. 0, falls ein Fehler beim Laden aufgetreten ist
        DLLEXPORT Schrift *ladeSchrift();
        //! Ldt eine einzelne Schriftgre aus der Datei
        //! \param schriftgre Die zu ladende Schriftgre
        //! \return Ein Alphabet mit den Zeichen in der Schriftgre. 0, falls die Schriftgre nicht gefunden wurde
        DLLEXPORT Alphabet *ladeAlphabet( int schriftgre );
        //! Ldt ein bestimmtes Zeichen einer bestimmten Schriftgre
        //! \param schriftgre Die Schriftgre, zu dem das Zeichen gehrt
        //! \param zeichen Der ASCII code des zu ladenden Zeichens
        //! \return Der geladene Buchstabe. 0, falls das Zeichen nicht gefunden wurde.
        DLLEXPORT Buchstabe *ladeBuchstabe( int schriftgre, unsigned char zeichen );
        //! Gibt den Pfad zur LTDS Datei zurck
        DLLEXPORT Text *getPfad() const;
        //! Gibt die Anzahl der gespeicherten Schriftgren zurck
        DLLEXPORT int getAnzahlSchriftgren() const;
        //! Gibt einen Array mit den gespeicherten Schriftgren zurck
        //! Der Array sollte nicht verndert werden
        DLLEXPORT unsigned char *getSchriftGren() const;
        //! Gibt die Anzahl an gespeicherten Zeichen in einer Schriftgre zurck
        //! \param sg Die Schriftgre, von der die Anzahl der ZEichen ermittelt werden soll
        //! \return Die Anzahl der Zeichen.
        DLLEXPORT unsigned char getAnzahlBuchstaben( int sg );
        //! Gibt einen Array mit Buchstaben einer bestimmten Schriftgre zurck
        //! \param sg Die Schriftgre
        //! \return Der Array mit den ASCII codes der Zeichen. 0, falls die Schriftgre nicht gefunden wurde.
        DLLEXPORT unsigned char *getBuchstaben( int sg );
    };
#endif
    //! Bit Funktionen 

    //! gibt 1-bits in gewinschter anzahl zurck.
    //! \param a Die Anzahl der Bits, die 1 sein sollen
    //! \return 32 Bits, wobei die einser Bits von rechts beginnen
    DLLEXPORT int Bits( int a );
    //! Gibt zurck, wie viele Bits bentigt werden um eine Zahl darzustellen
    //! \param c Die Zahl, die dargestellt werden soll
    //! \return Die Anzahl der bentigten Bits
    DLLEXPORT int getBits( char c );
}
#endif